﻿package implement

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/dao"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	zhios_order_relate_utils "code.fnuoos.com/EggPlanet/egg_models.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_models.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

func NewPublicPlatoonUserRelationDb(engine *xorm.Engine) dao.PublicPlatoonUserRelationDao {
	return &PublicPlatoonUserRelationDb{Db: engine}
}

type PublicPlatoonUserRelationDb struct {
	Db *xorm.Engine
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationInsert(PublicPlatoonUserRelation *model.PublicPlatoonUserRelation) (int64, error) {
	_, err := p.Db.InsertOne(PublicPlatoonUserRelation)
	if err != nil {
		return 0, err
	}
	return PublicPlatoonUserRelation.Id, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationUpdate(session *xorm.Session, id interface{}, PublicPlatoonUserRelation *model.PublicPlatoonUserRelation, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(PublicPlatoonUserRelation)
	} else {
		affected, err = session.Where("id=?", id).Update(PublicPlatoonUserRelation)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationGetOneByParams(params map[string]interface{}) (*model.PublicPlatoonUserRelation, error) {
	var m model.PublicPlatoonUserRelation
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := p.Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &m, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationGetOneByPid(recommendUid string, params map[string]interface{}) (*model.PublicPlatoonUserRelation, error) {
	var m model.PublicPlatoonUserRelation
	var query = fmt.Sprintf("%s <=?", params["key"])
	if has, err := p.Db.Where("recommend_uid = ?", recommendUid).And(query, params["value"]).OrderBy("id ASC").Get(&m); err != nil || has == false {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &m, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationFindByPid(fatherUid int64, fatherName, positionName string) ([]model.PublicPlatoonUserRelation, error) {
	var m []model.PublicPlatoonUserRelation
	var query2 = fmt.Sprintf("%s = ?", fatherName)
	var order = fmt.Sprintf("%s Desc", positionName)
	if err := p.Db.Where(query2, fatherUid).OrderBy(order).
		Find(&m); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return m, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationFindCountByPosition(fatherUid int64, fatherName, positionName string, startPosition, endPosition int64) ([]model.PublicPlatoonUserRelation, int64, error) {
	var m []model.PublicPlatoonUserRelation
	var count int64
	var query1 = fmt.Sprintf("%s >= ?", positionName)
	var query2 = fmt.Sprintf("%s <= ?", positionName)
	var query3 = fmt.Sprintf("%s = ?", fatherName)
	if count, err := p.Db.Where(query3, fatherUid).And(query1, startPosition).And(query2, endPosition).OrderBy("has_son_num ASC").
		FindAndCount(&m); err != nil {
		return nil, count, zhios_order_relate_logx.Warn(err)
	}
	return m, count, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationFindByParamsByPage(params map[string]interface{}, page, pageSize int) (*[]model.PublicPlatoonUserRelation, error) {
	var m []model.PublicPlatoonUserRelation
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := p.Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := p.Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := p.Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}
	}
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationFind(uid int64) ([]*model.PublicPlatoonUserRelation, error) {
	var head model.PublicPlatoonUserRelation
	if uid != 0 {
		has, err1 := p.Db.Where("uid = ?", uid).Get(&head)
		if err1 != nil {
			return nil, zhios_order_relate_logx.Error(err1)
		}
		if has == false {
			return nil, zhios_order_relate_logx.Error("未查询到数据")
		}
	} else {
		has, err2 := p.Db.Where("level_total = ?", 1).Get(&head)
		if err2 != nil {
			return nil, err2
		}
		if has == false {
			return nil, zhios_order_relate_logx.Error("未查询到数据")
		}
	}
	relations, err3 := p.GetTreeBySmall(&head, 3)
	if err3 != nil {
		return nil, err3
	}
	return relations, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationFindByUid(uid int64) ([]*model.PublicPlatoonUserRelation, error) {
	var head model.PublicPlatoonUserRelation
	has, err1 := p.Db.Where("uid = ?", uid).Get(&head)
	if err1 != nil {
		return nil, zhios_order_relate_logx.Error(err1)
	}
	if has == false {
		return nil, zhios_order_relate_logx.Error("未查询到数据")
	}
	relations, err3 := p.GetTreeBySmall(&head, 0)
	if err3 != nil {
		return nil, err3
	}
	return relations, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationGetOneByUid(uid int64) (*model.PublicPlatoonUserRelation, error) {
	var parent model.PublicPlatoonUserRelation
	has, err := p.Db.Where("uid = ?", uid).Get(&parent)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, zhios_order_relate_logx.Error("未查询到上级用户公排记录")
	}
	return &parent, nil
}

func (p PublicPlatoonUserRelationDb) PublicPlatoonUserRelationGetOneByPosition(position int64) (*model.PublicPlatoonUserRelation, error) {
	var relation model.PublicPlatoonUserRelation
	_, err := p.Db.Where("position = ?", position).Get(&relation)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &relation, nil
}

func (p PublicPlatoonUserRelationDb) GetTreeBySmall(head *model.PublicPlatoonUserRelation, leveLimit int) ([]*model.PublicPlatoonUserRelation, error) {
	if leveLimit == 0 {
		leveLimit = 1
	}
	var res []*model.PublicPlatoonUserRelation
	var queue []*model.PublicPlatoonUserRelation
	queue = append(queue, head)
	for len(queue) > 0 {
		node := queue[0]
		queue = queue[1:]
		if node.LevelTotal > head.LevelTotal+leveLimit {
			break
		}
		res = append(res, node)
		var son []*model.PublicPlatoonUserRelation
		err1 := p.Db.Where("father_uid1 = ?", node.Uid).Find(&son)
		if err1 != nil {
			return nil, zhios_order_relate_logx.Error(err1)
		}
		queue = append(queue, son...)
	}
	return res, nil
}
